from django.core import management
from django.core.management.base import BaseCommand
from django.core.management.commands.loaddata import Command as LoaddataCommand
from django.conf import settings
import os, itertools
from south.management.commands.syncdb import Command as SyncdbCommand
from south.management.commands.migrate import Command as MigrateCommand

class Command(BaseCommand):
    help = 'Create a new database and installs the named fixture(s) in the database.'
    args = "fixture [fixture ...]"
    def handle(self, *fixture_labels, **options):
        self.rename_database()

        management.call_command('syncdb', migrate_all=True, interactive=False)
        management.call_command('migrate', fake=True)
        if fixture_labels:
            management.call_command('loaddata', *fixture_labels)

    def rename_database(self):
        try:
            eng = settings.DATABASES['default']['ENGINE']
        except:
            return
        if 'sqlite3' in eng:
            db = settings.DATABASES['default']['NAME']
            if os.access(db, os.F_OK):
                newname = self.get_available_name(db)
                os.rename(db, newname)

    def get_available_name(self, name):
        """
        Returns a filename that's free on the target storage system, and
        available for new content to be written to.
        """
        dir_name, file_name = os.path.split(name)
        file_root, file_ext = os.path.splitext(file_name)
        # If the filename already exists, add an underscore and a number (before
        # the file extension, if one exists) to the filename until the generated
        # filename doesn't exist.
        count = itertools.count(1)
        while os.access(name, os.F_OK):
            # file_ext includes the dot.
            name = os.path.join(dir_name, "%s_%s%s" % (file_root, count.next(), file_ext))

        return name
