from analyses.main import analysesmodules
from django.contrib.auth.models import User
from django.core.management.base import NoArgsCommand, CommandError
from django.db import transaction
from utils.exchange import exchange

import logging
import os

try:
    import wx
    wx_available = True
except:
    wx_available = False

if wx_available:
    import wx.lib.filebrowsebutton as filebrowse

    class LogToText(logging.Handler):
      def __init__(self, textCtl):
          logging.Handler.__init__(self)
          self.textCtl = textCtl

      def emit(self, record):
          self.textCtl.AppendText(self.format(record))
          self.textCtl.AppendText('\n')

    class MyFrame(wx.Frame):
        """
        This is MyFrame.  It just shows a few controls on a wxPanel,
        and has a simple menu.
        """
        def __init__(self, parent, title):
            wx.Frame.__init__(self, parent, -1, title,
                              pos=(100, 100), size=(500, 500))
            self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

            # Create the menubar
            menuBar = wx.MenuBar()

            # and a menu
            menu = wx.Menu()

            # add an item to the menu, using \tKeyName automatically
            # creates an accelerator, the third param is some help text
            # that will show up in the statusbar
            menu.Append(wx.ID_EXIT, "E&xit\tAlt-X", "Exit the application")

            # bind the menu event to an event handler
            self.Bind(wx.EVT_MENU, self.OnCloseMe, id=wx.ID_EXIT)

            # and put the menu on the menubar
            menuBar.Append(menu, "&File")
            self.SetMenuBar(menuBar)

            # Now create the Panel to put the other controls on.
            panel = wx.Panel(self)

            # and a few controls
            text = wx.StaticText(panel, -1, "Select the new suggestionlists file")
            text.SetFont(wx.Font(14, wx.SWISS, wx.NORMAL, wx.BOLD))
            text.SetSize(text.GetBestSize())
            btn = wx.Button(panel, -1, "Replace")

            self.status = wx.StaticText(panel, -1, "")
            self.status.SetSize(text.GetBestSize())

            # bind the button events to handlers
            self.Bind(wx.EVT_BUTTON, self.Replace, btn)

            self.fbsf = filebrowse.FileBrowseButton(
                panel, -1, size=(450, -1),
                labelText = 'Suggestionlists file:', fileMask = '*.json'
                )

            # Set up a log window
            self.logbox = wx.TextCtrl(panel, -1, size=(450,270),
                                  style = wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)

            sizer = wx.BoxSizer(wx.VERTICAL)
            sizer.Add(text, 0, wx.ALL, 5)
            sizer.Add(self.fbsf, 0, wx.ALL, 5)
            sizer.Add(btn, 0, wx.ALL, 5)
            sizer.Add(self.status, 0, wx.ALL, 5)
            sizer.Add(self.logbox, 0, wx.ALL, 5)
            box = wx.BoxSizer()
            box.Add(sizer, 0, wx.ALL, 20)
            panel.SetSizer(box)
            panel.Layout()

        def OnCloseWindow(self, event):
            self.Destroy()
        def OnCloseMe(self, event):
            self.Close(True)

        def Replace(self, evt):
            """Event handler for the button click."""
            sffile = self.fbsf.GetValue()
            if not sffile:
                self.status.SetLabel('You must select a file.')
            else:
                self.status.SetLabel('Replacing the system suggestionlists.')

               # create logger
                logger = logging.getLogger()
                logger.setLevel(logging.DEBUG)
                # create console handler and set level to debug
                ch = LogToText(self.logbox)
                ch.setLevel(logging.DEBUG)
                formatter = logging.Formatter("%(message)s")
                # add formatter to ch
                ch.setFormatter(formatter)
                # add ch to logger
                logger.addHandler(ch)

                replace(sffile)
                self.status.SetLabel('Replacement finished')


    class MyApp(wx.App):
        def OnInit(self):
            frame = MyFrame(None, "Planop 3 replace system suggestionlists")
            self.SetTopWindow(frame)
            frame.Show(True)
            return True

class Command(NoArgsCommand):

    def handle_noargs(self, **options):
        if wx_available:
            app = MyApp(False)
            app.MainLoop()
        else:
            print "Error: wx package not available"

@transaction.commit_on_success
def replace(fixture_label):
    logger = logging.getLogger()
    output = []
    try:
        user = User.objects.get(username='__SYSTEM__')
    except User.DoesNotExist:
        raise CommandError("System user not found")
    for module in analysesmodules.get_items_sorted():
        logger.debug(u'removing system suggestion lists for %s' % module['appconfig'].analysistype)
        for item in module['appconfig'].get_model('suggestionlist').objects.filter(system=True):
            item.delete(user, override=True)
        for item in module['appconfig'].get_model('apsuggestionlist').objects.filter(system=True):
            item.delete(user, override=True)
    load = True
    try:
        fixture = open(fixture_label)
    except:
        logger.debug(u'Error opening fixture %s' % fixture_label)
        load = False
    if load:
        try:
            logger.debug(u'loading fixture %s' % fixture_label)
            exchange.load(user, fixture.read(), set_dirty=False)
            logger.debug(u'finished loading fixture')
        except:
            logger.debug(u'Error loading fixture %s' % fixture_label)
        fixture.close()
