"""
Helpers for report generation

"""

from django.conf import settings
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.shortcuts import render_to_response
##from django.http import HttpResponse
from django.template import loader, Context, Template
##from django.utils.encoding import smart_str
from django.utils import translation
from django.utils.translation import ugettext as _
from django.utils.safestring import mark_safe

from email.MIMEImage import MIMEImage

from xhtml2pdf import pisa
import cStringIO as StringIO

TESTPDF = False
#TESTPDF = True

##def Report(request, context, template, filename):
##    """
##    View that returns an rtf document.
##    """
##    response = HttpResponse(content_type='application/rtf')
##    response['Content-Disposition'] = 'attachment; filename=%s' % filename
##    t = loader.get_template('%s' % template)
##    c = Context(context)
##    response.write(smart_str(t.render(c), encoding='iso-8859-1', errors='replace'))
##    return response

def _get_context(language):
    return {
        'STATIC_ROOT': settings.STATIC_ROOT,
        'PROJECTNAME': settings.PROJECTNAME,
        'LANGUAGES': settings.LANGUAGES,
        'LANGUAGE_CODE': language,
    }


def make_pdf(body_strings, css_files, language=u'en', outertemplate='outerreport.html'):
    cur_language = translation.get_language()
    try:
        translation.activate(language)
    except:
        pass

    context = _get_context(language)
    body = mark_safe(u'\n'.join(body_strings))
    mycss = ['report.css',]
    mycss.extend(css_files)
    # eliminate duplicates
    seen = {}
    css = []
    for item in mycss:
       if item in seen: continue
       seen[item] = 1
       css.append(item)
    style = mark_safe(u'\n'.join([loader.render_to_string(item, Context(context)) for item in css]))
    context.update({
        'style': style,
        'body': body,
    })
    html = loader.render_to_string(outertemplate, Context(context))
    translation.activate(cur_language)

    # for testing
    if TESTPDF:
        return html

    result = StringIO.StringIO()
    source = StringIO.StringIO(html.encode("ISO-8859-1", 'replace'))
    pdf = pisa.pisaDocument(source, result)
    if not pdf.err:
        return result.getvalue()
    raise Exception('There was an error creating the report.')

def make_pdf_body(context, language=u'en', template='crud/report.html'):
    extra_context = _get_context(language)
    extra_context.update(context)
    cur_language = translation.get_language()
    try:
        translation.activate(language)
    except:
        pass
    body = loader.render_to_string(template, Context(extra_context))
    translation.activate(cur_language)
    return body

def send_mail(pdf, user, language=u'en', reportname='report', reference=''):
    # send mail
    subject = _('Planop report')
    if reference:
        subject = u'%s %s' % (subject, reference)
    context = _get_context(language)
    context['reference'] = reference
    context = Context(context)
    text = loader.get_template('reportmail.txt')
    html = loader.get_template('reportmail.html')
    cur_language = translation.get_language()
    try:
        translation.activate(language)
    except:
        pass
    text_content = text.render(context)
    html_content = html.render(context)
    translation.activate(cur_language)

    email = EmailMultiAlternatives(
        subject,
        text_content,
        settings.REPORTS_EMAIL,
        [user.email,],
    )
    email.attach_alternative(html_content, "text/html")
    email.attach('%s.pdf' % reportname, pdf, 'application/pdf')
    email.send()

