# search views

from django.core.paginator import Paginator, InvalidPage
from django import forms
from django.utils.translation import ugettext_lazy as _

from utils.search import models as app_models
from utils.search import forms as app_forms
from utils.search import tables as app_tables
from utils.versioning import versionedcrud

class SearchFilter(object):
    form = app_forms.SearchForm
    prefix = 'filter'

    def __call__(self, request, queryset, version, maxitems):
        filterform = None
        results = []
        filtered = False
        if self.form:
            if request.method == 'POST':
                if request.POST.get('submit_clear'):
                    filterform = self.form(prefix=self.prefix)
                else:
                    filterform = self.form(data=request.POST, prefix=self.prefix)
                    if filterform.is_valid():
                        results, filtered = self.search(filterform, request.user, version, maxitems)
            else:
                filterform = self.form(prefix=self.prefix)
        return results, filterform, filtered


    def search(self, filterform, user, version, maxitems):
        """
        populate results from app queries
        """
        filtered = False
        query = filterform.cleaned_data.get('query','')
        if query:
            filtered=True
        objecttype = filterform.cleaned_data.get('objecttype')
        searches = app_models.searches.get_searches(objecttype)
        results = []
        for search in searches:
            if maxitems:
                max = maxitems - len(results) + 1
            else:
                max = None
            results += search().search(query, user, version, max)
            if maxitems and len(results) >= maxitems:
                break
        return results, filtered


class search(versionedcrud.TableListView):
    "Search"

    table_class = app_tables.ResultsTable
    defaultsort = 'duedate'
    paginate_by = 100
    title = _('Search')

    def __call__(self, request, *args, **kwargs):
        self.request = request

        self.setup_from_args(*args, **kwargs)

        queryset = None

        maxitems = None
        if self.paginate_by:
            page_number = self.get_page_number(request)
##            maxitems = self.paginate_by * page_number
        queryset, filter_form, filtered = SearchFilter()(request, queryset, self.version, maxitems)
        table = self.get_table(request, queryset)
        if self.paginate_by:
            page_number = self.get_page_number(request)
            table.paginate(Paginator, self.paginate_by, page=page_number,
                allow_empty_first_page=self.allow_empty)

        media = forms.widgets.Media()
        if filter_form:
            media = media + filter_form.media

        extra_context = {
            'table': table,
            'title': self.title,
            'currentmenuitem': self.currentmenuitem,
            'filterform': filter_form,
            'media': media,
            'csv': bool(self.csv_table_class)
        }
        self.add_to_extra_context(extra_context)
        return self.get_response(extra_context)

    def get_table(self, request, queryset):
        if self.table_class:
            order_by = self.request.GET.get('sort',self.defaultsort)
            return self.table_class(data=queryset, order_by=order_by)
        return None

    def add_to_extra_context(self, extra_context):
        super(search, self).add_to_extra_context(extra_context)

