from utils.versioning.models import Version, WorkVersion

def get_current_version(request):
    """
    Returns the active version + a boolean indicating whether that version
    was deliberately activated.
    If the user activated a version (version_id in the session),
    return the active version + True.
    If the user has permission on the Workversion: return the Workversion,
    otherwise return the last public version.
    In this last case returns None if no public version is available.

    """
    if request.session.get('version_id', None):
        try:
            v = Version.objects.get(pk=request.session['version_id'])
            return (v, True)
        except Version.DoesNotExist:
            # invalid version_id in session
            pass
    if request.user.has_perm('versioning.display_versions'):
        return (WorkVersion, False)
    else:
        v = Version.objects.filter(published=True)
        if v:
            return (v[0], False)
    return (None, False)
